#!/usr/bin/env python

'''KHMedia applications.
Includes scheduled recorder, music player, sound level meter and a webcam viewer
'''

from distutils.core import setup
from distutils.extension import Extension
from glob import glob
from mediaserver import VERSION

# patch distutils if it's too old to cope with the "classifiers" or
# "download_url" keywords
from sys import version
if version < '2.2.3':
    from distutils.dist import DistributionMetadata
    DistributionMetadata.classifiers = None
    DistributionMetadata.download_url = None

if __name__ == '__main__':
    setup(
        name = 'mediaserver',
        version = VERSION.version_string,
        description = 'Mediacore mediaserver',
        long_description = __doc__,
        author = 'Miqra Engineering',
        author_email='packaging@miqra.nl',
        maintainer = 'Miqra Engineering Packaging',
        maintainer_email = 'packaging@miqra.nl',
        license='',
        platforms=['posix'],
        url='',
        classifiers = [
            'Development Status :: 5 - Production/Stable',
            'Intended Audience :: End Users/Desktop',
            'Operating System :: POSIX :: Linux',
            'Programming Language :: Python :: 2',
            'Topic :: Multimedia :: Sound/Audio',
            'Topic :: Multimedia :: Sound/Audio :: Players',
            ],
        packages=['mediaserver'],
        package_data={'mediaserver': ['image/*', ]},
        data_files = [
            # note that some files are forced to /usr/share/... instead of just share/..
            # this is because the system does not look in /usr/local/share/... for those files, but only in /usr/share/...        
            ('/usr/share/mediaserver', glob('usr-share-mediaserver/*')),
            ('/etc/dbus-1/system.d', glob('etc/dbus-1/system.d/*')),
            ('/etc/init.d', glob('etc/init.d/*')),
            ('bin', glob('bin/*')),
            ('sbin', glob('sbin/*')),
            ('/usr/share/mediaserver/glimagesink_so',glob('glimagesink_so/*')),
            ],
    )
